#!/usr/bin/env python

import collections, sys, getopt, os, errno, json, subprocess, time, codecs

content_path = None
key = None
value = None

def read_json(full_path):
    with open(full_path, 'r') as file:
        return json.load(file)

def write_json(full_path, object):
    with codecs.open(full_path, 'w', encoding="utf-8") as file:
        json.dump(object, file, sort_keys = True, indent = 4, ensure_ascii = False)
    os.chmod(full_path, 0o600)

try:
    opts, args = getopt.getopt(sys.argv[1:], 'p:k:v:', ['path=', 'key=', 'value='])

    for k, v in opts:
        if k == '-p' or k == '--path':
            content_path = v
        if k == '-k' or k == '--key':
            key = v
        if k == '-v' or k == '--value':
            value = v
except getopt.GetoptError as e:
    print(e)
    sys.exit(2)

if not content_path:
    print("--path is required argument")
    sys.exit(2)

if not key:
    print("--key is required argument")
    sys.exit(2)

content = {}
is_content_modified = False

if os.path.exists(content_path):
    content = read_json(content_path)

node = content
keys = key.split(".")
depth = len(keys)
for i, k in enumerate(keys):
    if i == depth - 1:
        if value and (not k in node or node[k] != value):
            # set
            node[k] = value
            is_content_modified = True
        elif k in node:
            # get
            print(node[k])
    else:
        # traverse object tree
        if not k in node:
            node[k] = {}
        node = node.get(k, None)

if is_content_modified:
    write_json(content_path, content)
